/****************************************************************
 * purgebackups Mark I 					*
 *								*
 * $Revision: 1.1 $						*
 * $Date: 2009/10/26 10:40:23 $					*
 *								*
 ****************************************************************/

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <strings.h>
#include <pwd.h>
#include <errno.h>
#include <time.h>

#define DIR_PROC_DELAY 10L

extern int errno;

static char rcsid[] = "$Id: purgebackups.c,v 1.1 2009/10/26 10:40:23 root Exp $";

static int debug = 0;

#define DEF_CONFIGFILE "/usr/local/admin/etc/inbox-backup.conf"
#define LOCKFILE "/tmp/bkup-inboxes.lck"

#define MAXLINELEN	1024

#define DEF_USERHOME	"/home/users"
#define DEF_BACKUPDIR	"/home/backup-inboxes"
#define DEF_MAXAGE	90*24*60*60 /* 90 days */

struct config {
	char	cfg_userhome[MAXNAMLEN];
	char	cfg_backupdir[MAXNAMLEN];
	time_t	cfg_maxage;
};

int readconfig(struct config *cfg);
int unlinkoldfiles(struct config cfg);

void version(char *prognam);
void usage(char *prognam);

static char configfile[MAXNAMLEN] = DEF_CONFIGFILE;

int main(int argc, char **argv)
{
	struct config cfg;
	int retval = 0, c, fd;
	extern char *optarg;
	extern int optind, optopt;

	while ((c = getopt(argc, argv, "vVd:C:")) != -1)
	{
		switch(c)
		{
			case 'v':
				version(argv[0]);
				break;
			case 'd':
				debug = atoi(optarg);
				break;
			case 'C':
				strncpy(configfile, optarg, MAXNAMLEN - 1);
				configfile[MAXNAMLEN - 1] = NULL;
				break;
			case ':':
				fprintf(stderr, "Option -%c requires an value\n", optopt);
			case 'V':
				version(argv[0]);
			case '?':
				usage(argv[0]);
				retval = -1;
		}
	}

	if (access(LOCKFILE, R_OK) == 0)
	{
		if (debug > 0)
			fprintf(stderr, "Lockfile exists, exiting.\n");
		return -1;
	}

	if ((fd = creat(LOCKFILE, 0600)) != -1)
		close(fd);

	if (retval == 0)
		if ((retval = readconfig(&cfg)) == 0)
			retval = unlinkoldfiles(cfg);

	unlink(LOCKFILE);

	return(retval);
}

int readconfig(struct config *cfg)
{
	FILE *fp;
	char buffer[MAXLINELEN + 1], option[10], value[512];
	int i, nval, maxage, retval = 0;

	if ((fp = fopen(configfile, "r")) == NULL)
	{
		strcpy(cfg->cfg_userhome, DEF_USERHOME);
		strcpy(cfg->cfg_backupdir, DEF_BACKUPDIR);
		cfg->cfg_maxage = (time_t) DEF_MAXAGE;
	}
	else
	{
		for (i = 1; fgets(buffer, MAXLINELEN, fp) != NULL ; i++)
		{
			nval = sscanf(buffer, "%9s %511s", option, value);

			if (*option != '#')
			{
				if (nval < 2)
				{
					fprintf(stderr, "Syntax error at line %d in configuration file: only one item on line\n", i);
					retval = -1;
					goto end;
				}

				if (!strcasecmp("userhome", option))
				{
					strcpy(cfg->cfg_userhome, value);
				}
				else if (!strcasecmp("backupdir", option))
				{
					strcpy(cfg->cfg_backupdir, value);
				}
				else if (!strcasecmp("maxage", option))
				{
					if ((maxage = atoi(value)) == 0)
					{
						fprintf(stderr, "Syntax error at line %d in configuration file: non-integer value given for maxage: %s\n", i, value);
						retval = -1;
						goto end;
					}

					cfg->cfg_maxage = maxage * 24 * 60 * 60;
				}
				else
				{
					fprintf(stderr, "Syntax error at line %d in configuration file: unknown keyword: %s\n", i, option);
					retval = -1;
					goto end;
				}
			}
		}
	}
end:
	fclose(fp);

	return retval;
}

int unlinkoldfiles(struct config cfg)
{
	DIR *maind, *userd;
	char msgdirectory[MAXNAMLEN], msgfilename[MAXNAMLEN];
	struct dirent *mainent, *userent;
	struct stat filestat;
	time_t oldage;

	oldage = time(NULL) - cfg.cfg_maxage;

	if ((maind = opendir(cfg.cfg_backupdir)) == NULL)
	{
		fprintf(stderr, "unlinkoldfiles: can't open mail backup directory: %s: %s\n", cfg.cfg_backupdir, strerror(errno));
		return errno;
	}

	while((mainent = readdir(maind)) != NULL)
	{
		if (debug > 2) fprintf(stderr, "unlinkoldfiles: debug: examining %s/%s\n", cfg.cfg_backupdir, mainent->d_name);

		if (*mainent->d_name == '.')
			continue;

		if (debug > 4) fprintf(stderr, "unlinkoldfiles: debug: not a dot file\n");

		sprintf(msgdirectory, "%s/%s", cfg.cfg_backupdir, mainent->d_name);

		if ((userd = opendir(msgdirectory)) == NULL)
		{
			fprintf(stderr, "unlinkoldfiles: failed to open user's backup directory: %s: %s\n", msgdirectory, strerror(errno));
			continue;
		}
		else
		{
			if (debug > 2) fprintf(stderr, "unlinkoldfiles: debug: searching for mailfiles.\n");

			while((userent = readdir(userd)) != NULL)
			{
				if (debug > 3) fprintf(stderr, "unlinkoldfiles: debug: found %s\n", userent->d_name);

				if (*userent->d_name == '.')
					continue;

				if (debug > 4) fprintf(stderr, "unlinkoldfiles: debug: not a dot file.\n");

				sprintf(msgfilename, "%s/%s", msgdirectory, userent->d_name);

				if (stat(msgfilename, &filestat) == 0)
				{
					if (filestat.st_atime < oldage)
					{
						if (debug > 4) fprintf(stderr, "unlinkoldfiles: debug: file untouched for more than %ld days, deleting.\n", cfg.cfg_maxage / (24 * 60 * 60));

						unlink(msgfilename);
					}
					else
					{
						if (debug > 4) fprintf(stderr, "unlinkoldfiles: debug: file accessed within %ld days, ignoring.\n", cfg.cfg_maxage / (24 * 60 * 60));
					}
				}
				else
				{
					fprintf(stderr, "unlinkoldfiles: can't stat %s: %s\n", msgfilename, strerror(errno));
				}
			}
		}
		closedir(userd);
	}
	closedir(maind);
	return 0;
}

void version(char *prognam)
{
	fprintf(stderr, "%s: %s\n", prognam, rcsid);
}

void usage(char *prognam)
{
	fprintf(stderr, "Usage: %s [-V] [-v] [-C configfile] [-d debuglevel]\n", prognam);
}
